﻿using System;

namespace _01_Shell
{
    //客户端程序
    class Program
    {
        static void Main(string[] args)
        {
            ConcretePrototype1 p1 = new ConcretePrototype1("I");            //原型对象(来自外部的第一个实例)
            ConcretePrototype1 c1 = (ConcretePrototype1)p1.Clone();        //浅拷贝(shallow copy)
            Console.WriteLine("Cloned: {0}", c1.Id);

            ConcretePrototype2 p2 = new ConcretePrototype2("II");           //原型对象(来自外部的第一个实例)
            ConcretePrototype2 c2 = (ConcretePrototype2)p2.Clone();        //浅拷贝(shallow copy)
            Console.WriteLine("Cloned: {0}", c2.Id);
                        
            Console.Read();
        }
    }

    //每个具体的「原型」类，要实现的抽象类或接口。
    //亦可舍弃此一自定义抽象类或接口，让 ConcretePrototype 类改为实现 .NET 原生的 System.ICloneable 接口
    abstract class Prototype
    {
        private string id;                                    //这个字段在派生类的 Clone 方法被调用时，会自动被拷贝
                
        public Prototype(string id)                       //构造函数
        {
            this.id = id;
        }

        public string Id
        {
            get { return id; }
        }

        //返回 Prototype 类型。
        public abstract Prototype Clone();
    }

    class ConcretePrototype1 : Prototype
    {
        public ConcretePrototype1(string id)         //构造函数
            : base(id)
        {
        }

        //返回 Prototype 类型。在客户端程序中，依赖和获得的是 Prototype 抽象类，并以其定义来操作其派生类
        public override Prototype Clone()
        {
            return (Prototype)this.MemberwiseClone();          //浅拷贝(shallow copy)
        }
    }


    class ConcretePrototype2 : Prototype
    {
        public ConcretePrototype2(string id)        //构造函数
            : base(id)
        {
        }

        //返回 Prototype 类型。在客户端程序中，依赖和获得的是 Prototype 抽象类，并以其定义来操作其派生类
        public override Prototype Clone()
        {
            return (Prototype)this.MemberwiseClone();       //浅拷贝(shallow copy)
        }
    }
}

/*
执行结果：

Cloned: I
Cloned: II

*/
//
