﻿using System;
using com.cnblogs.WizardWu.Sample01;

//客户端程序
public partial class _01_ShallowCopy : System.Web.UI.Page
{
    protected void Page_Load(object sender, EventArgs e)
    {
        Developer dev = new Developer();          //原型对象(来自外部的第一个实例)
        dev.Name = "王小明";
        dev.Role = "资深工程师";
        dev.PreferredLanguage = "C#";

        Developer devCopy1 = (Developer)dev.Clone();      //浅拷贝(shallow copy)
        devCopy1.Name = "李大同";

        Developer devCopy2 = (Developer)dev.Clone();      //浅拷贝(shallow copy)
        devCopy2.Name = "吴宇泽";
        devCopy2.Role = "研发工程师";
        devCopy2.PreferredLanguage = "C++";

        Response.Write(dev + "<br>");
        Response.Write(devCopy1 + "<br>");
        Response.Write(devCopy2 + "<p>");

        /* 执行结果：

        王小明 - 资深工程师 - C#
        李大同 - 资深工程师 - C#
        吴宇泽 - 研发工程师 - C++

        */

        Typist typist = new Typist();                    //原型对象(来自外部的第一个实例)
        typist.Name = "左婉青";
        typist.Role = "行政文员";
        typist.WordsPerMinute = 120;

        Typist typistCopy1 = (Typist)typist.Clone();            //浅拷贝(shallow copy)
        typistCopy1.Name = "周玉婷";
        typistCopy1.WordsPerMinute = 115;

        Response.Write(typist + "<br>");
        Response.Write(typistCopy1 + "<br>");

        /* 执行结果：

        左婉青 - 行政文员 - 120 字/分
        周玉婷 - 行政文员 - 115 字/分

        */
    }
}

//服务器端程序
namespace com.cnblogs.WizardWu.Sample01
{
    //员工 抽象类。
    //每个具体的「原型」类，要实现的抽象类或接口
    abstract class Employee
    {
        //返回 Employee 类型。
        public abstract Employee Clone();

        //.NET 3.0 的 Auto-Implemented Property (Automatic Properties) 语法，会自动产生对应的同名 private field
        public string Name { get; set; }       //姓名
        public string Role { get; set; }         //职务
    }

    //程序员
    //继承 Employee 抽象类，或实现 .NET 的 ICloneable 接口，以便重写其 Clone 方法，创建作为当前实例副本的新对象
    class Developer : Employee
    {
        public string PreferredLanguage { get; set; }

        //返回 Employee 类型。在客户端程序中，依赖和获得的是 Prototype 抽象类，并以其定义来操作其派生类
        public override Employee Clone()
        {
            return (Employee)MemberwiseClone();        //浅拷贝(shallow copy)。转型亦可用 as Employee
            //return new Developer();                           //深拷贝(deep copy)
        }

        public override string ToString()
        {
            return string.Format("{0} - {1} - {2}", Name, Role, PreferredLanguage);
        }
    }

    //文员
    //继承 Employee 抽象类，或实现 .NET 的 ICloneable 接口，以便重写其 Clone 方法，创建作为当前实例副本的新对象
    class Typist : Employee
    {
        public int WordsPerMinute { get; set; }

        //返回 Employee 类型。在客户端程序中，依赖和获得的是 Prototype 抽象类，并以其定义来操作其派生类
        public override Employee Clone()
        {
            return (Employee)MemberwiseClone();        //浅拷贝(shallow copy)。转型亦可用 as Employee
            //return new Typist();                                 //深拷贝(deep copy)
        }

        public override string ToString()
        {
            return string.Format("{0} - {1} - {2} 字/分", Name, Role, WordsPerMinute);
        }
    }    

} //end of namespace